#ifndef GUARD_fileentry_h
#define GUARD_fileentry_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file fileentry.h
/// @brief Class for filename entry and browse button header file.

#include <gtkmm/box.h>
#include <gtkmm/entry.h>
#include <gtkmm/button.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/filechooserdialog.h>

#include <libintl.h>

/// @brief Class for filename entry and browse button.
/// @todo Replace it to FileChooserButton when change to gtkmm 2.5.x over(coressponding Gtk+ 2.6 over).
class FileEntry : public Gtk::HBox
{
    public:
	FileEntry(Gtk::Window& parentWnd, const Glib::ustring& tooltipText)
	: parentWnd_(parentWnd),
	browseButton_(gettext("Browse..."))
	{
	    entryTooltips_.set_tip(filenameEntry_, tooltipText);

	    browseButton_.signal_clicked().connect(
		    sigc::mem_fun(*this, &FileEntry::OnBtnBrowse) );

	    pack_start(filenameEntry_, Gtk::PACK_SHRINK);
	    pack_start(browseButton_, Gtk::PACK_SHRINK);
	}
	virtual ~FileEntry() {}

	void SetEntryText(const Glib::ustring& text)
	{
	    filenameEntry_.set_text(text);
	}

	Glib::ustring GetEntryText() const
	{
	    return filenameEntry_.get_text();
	}

    private:
	Gtk::Window& parentWnd_;
	Gtk::Entry filenameEntry_;
	Gtk::Button browseButton_;
	Gtk::Tooltips entryTooltips_;

	void OnBtnBrowse();

	virtual void on_size_request(Gtk::Requisition* requisition);

	virtual void on_size_allocate(Gtk::Allocation& allocation);
};

#endif

