#ifndef GUARD_player_ogg_h
#define GUARD_player_ogg_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file player_ogg.h
/// @brief Class for play ogg header file.

#include <vorbis/vorbisfile.h>

#include <glibmm.h>

#include "player.h"

class MainManager;

namespace VORBIS {

    /// @brief Class for play ogg.
    class OGGPlayer : public Player
    {
	public:
	    OGGPlayer(MainManager& mainManager);
	    virtual ~OGGPlayer();

	    virtual bool Init(const std::string& filename);
	    virtual void Start();
	    virtual void Stop();
	    virtual void Seek(const unsigned long& seconds);
	    virtual const long int& CurrentTime() const;
	    
	private:
	    OGGPlayer& operator=(const OGGPlayer& oggPlayer);
	    OGGPlayer(const OGGPlayer& oggPlayer);
	    
	    float **buf_;			    ///< Buffer to store decoded samples.

	    Glib::Dispatcher signalFinish_;	    ///< Dispatcher for finish signal. 
	    Glib::Dispatcher signalTimerUpdate_;    ///< Dispatcher for time update signal.

	    Glib::Mutex vfMutex_;		    ///< Mutex to protect vorbisFile_. 
	    
	    OggVorbis_File vorbisFile_; 
	    
	    DecodeState decodeState_;		    ///< DecodeState to check status.

	    long int currentSeconds_;		    ///< Current decoded time.
	    const int buflen_;			    ///< Decoded buffer size.
	    int sampleRate_;			    ///< Current sample rate.
	    int ret_;				    ///< Return variable of ov_read_*.
	    int bs_;				    ///< Current logical bitstream number in ov_read_*.

	    bool ovOpenSuccess_;		    ///< Success of ov_open.
	    
	    MainManager& mainManager_;		    ///< Program control MainManager.
	    
	    FILE* inFile_;			    ///< File descriptor to play. 
	    
	    virtual void Clean();
    };

}

std::auto_ptr<Player> CreateOGGPlayer(MainManager& mainManager);

#endif

