#ifndef GUARD_sockethandler_h
#define GUARD_sockethandler_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file sockethandler.h
/// @brief Class for unix socket handler header file.

#include <vector>
#include <string>

#include "socket.h"

class MainManager;

/// @brief Class for unix socket handler.
class UnixSocketHandler : public UnixSocket
{
    public:
	UnixSocketHandler(const std::vector<std::string>& arguments);
	virtual ~UnixSocketHandler();

	void SendArguments();
	void StartServerLoop();
	
	/// @brief Copy received data to dest and empty received data container.
	/// @param dest Destination to copy received data
	void ReceivedData(std::vector<std::string>& dest)
	{
	    Glib::Mutex::Lock lock(acceptMutex_);
	    
	    dest = receivedData_;
	    
	    std::vector<std::string>().swap(receivedData_);
	}

	void SetMainManager(MainManager* const mainManager);

    private:
	UnixSocketHandler(const UnixSocketHandler& unixSocketHandler);
	UnixSocketHandler& operator=(const UnixSocketHandler& unixSocketHandler);
	
	const std::vector<std::string>& arguments_; ///< Command line arguments.
	std::vector<std::string> receivedData_;	    ///< A container to store received data.
	
	Glib::Mutex acceptMutex_;		    ///< A mutex variable to protect container.
	Glib::Dispatcher signalReceived_;	    ///< A dispatcher variable for signal to MainManager.

	MainManager* mainManager_;

	virtual void DataReceived(const int& fd, const int& receivedBytes, char* buf, const int& bufSize);
};

#endif

