// Sagua is music player. 
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file fileinfodlg.cc
/// @brief Class for display file's tag and some information source file. 

#include <libintl.h>

#include <gtkmm/box.h>
#include <gtkmm/stock.h>

#include "../include/fileinfodlg.h"

/// @brief Constructor of FileInfoDlg.
/// @param fileInfo File information map.
/// @todo Display some message when there is no information correspond to section. 
FileInfoDlg::FileInfoDlg(std::map<std::string, Glib::ustring>& fileInfo)
: mainTable_( 4, 8),
fileInfoHead_( gettext("<span weight=\"bold\">File Information</span>"), 0.0, 0.5),
titleHead_( gettext("Title : "), 0.0, 0.5),
artistHead_( gettext("Artist : "), 0.0, 0.5),
albumHead_( gettext("Album : "), 0.0, 0.5),
trackHead_( gettext("Track : "), 0.0, 0.5),
yearHead_( gettext("Year : "), 0.0, 0.5),
genreHead_( gettext("Gern : "), 0.0, 0.5),
commentHead_( gettext("Comment : "), 0.0, 0.5),
title_( fileInfo["title"], 0.0, 0.5),
artist_( fileInfo["artist"], 0.0, 0.5),
album_( fileInfo["album"], 0.0, 0.5),
track_( fileInfo["track"], 0.0, 0.5),
year_( fileInfo["year"], 0.0, 0.5),
genre_( fileInfo["genre"], 0.0, 0.5),
comment_( fileInfo["comment"], 0.0, 0.5)
{
    set_title( gettext("File Information") );
    set_resizable(false);
  
    Gtk::VBox* mainBox_ = get_vbox();
   
    mainBox_->set_spacing(12); 
    mainBox_->set_border_width(12);
    mainBox_->pack_start(mainTable_);

    mainTable_.set_border_width(12);
    mainTable_.set_row_spacings(6);
    mainTable_.set_col_spacing(0, 12);
    mainTable_.set_col_spacing(1, 12);
    mainTable_.set_col_spacing(2, 12);

    fileInfoHead_.set_use_markup(true);
    mainTable_.attach(fileInfoHead_, 0, 4, 0, 1, Gtk::FILL);
    
    mainTable_.attach(titleHead_, 1, 2, 1, 2, Gtk::FILL);
    mainTable_.attach(title_, 3, 4, 1, 2, Gtk::FILL); 

    mainTable_.attach(artistHead_, 1, 2, 2, 3, Gtk::FILL);
    mainTable_.attach(artist_, 3, 4, 2, 3, Gtk::FILL);

    mainTable_.attach(albumHead_, 1, 2, 3, 4, Gtk::FILL);
    mainTable_.attach(album_, 3, 4, 3, 4, Gtk::FILL);

    mainTable_.attach(trackHead_, 1, 2, 4, 5, Gtk::FILL);
    mainTable_.attach(track_, 3, 4, 4, 5, Gtk::FILL);

    mainTable_.attach(yearHead_, 1, 2, 5, 6, Gtk::FILL);
    mainTable_.attach(year_, 3, 4, 5, 6, Gtk::FILL);

    mainTable_.attach(genreHead_, 1, 2, 6, 7, Gtk::FILL);
    mainTable_.attach(genre_, 3, 4, 6, 7, Gtk::FILL);

    mainTable_.attach(commentHead_, 1, 2, 7, 8, Gtk::FILL);
    mainTable_.attach(comment_, 3, 4, 7, 8, Gtk::FILL);

    Gtk::Button* okBtn = add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    okBtn->signal_clicked().connect(
	sigc::mem_fun(*this, &FileInfoDlg::OnBtnOk) );

    show_all_children();
}

FileInfoDlg::~FileInfoDlg()
{
}

/// @brief Callback function when press button OK.
///
/// Just close dialog.
void FileInfoDlg::OnBtnOk()
{
    hide();
}

