// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file mainwin.cc
/// @brief Class for main window source file.

#include <iostream>
#include <fstream>

#include <cstdlib>
#include <string>
#include <vector>
#include <map>

#include <gdkmm.h>
#include <gtkmm/main.h>
#include <gtkmm/scrollbar.h>
#include <gtkmm/filechooserdialog.h>

#include <libintl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "../include/mainwin.h"
#include "../include/mainmanager.h"
#include "../include/prefwin.h"
#include "../include/player_mp3.h"
#include "../include/player_ogg.h"
#include "../include/player_wma.h"

#ifdef ENABLE_FAAD2
#include "../include/player_aac.h"
#endif

#include "../include/playlist.h"
#include "../include/dirbrowsedlg.h"
#include "../include/dirplaywin.h"
#include "../include/misc.h"
#include "../include/trace.h"

MainWnd::MainWnd(MainManager& mainManager, const std::vector<std::string>& arguments)
    : mainManager_(mainManager),
    playerFactory_(new PlayerFactory),
    playlist_(NULL),
    randomPlayMenu_(gettext("_Random play"), true),
    repeatPlayMenu_(gettext("R_epeat play"), true),
    compactPlayMenu_(gettext("_Compact mode"), true),
    mainlistPlayMenu_(viewMenuGroup_, gettext("Main _list mode"), true),
    dirPlayMenu_(viewMenuGroup_, gettext("_Directory mode"), true),
    titleDisplayAlignment_(0.5, 0.5, 0.85, 1.0),
    seekScaleAlignment_(0.5, 0.5, 0.85, 1.0),
    volumnAlignment_(0.5, 0.5, 1.0, 0.62),
    seekScale_(0.0, 0.0, 1.0),
    volumnScale_(0.0, 100.1, 1.0),
    playImgBtn_( PIXMAPSDIR "play.png" , PIXMAPSDIR "play_o.png", PIXMAPSDIR "play_p.png"),
    pauseImgBtn_( PIXMAPSDIR "pause.png" , PIXMAPSDIR "pause_o.png", PIXMAPSDIR "pause_p.png"),
    stopImgBtn_( PIXMAPSDIR "stop.png" , PIXMAPSDIR "stop_o.png", PIXMAPSDIR "stop_p.png"),
    prevImgBtn_( PIXMAPSDIR "prev.png" , PIXMAPSDIR "prev_o.png", PIXMAPSDIR "prev_p.png"),
    nextImgBtn_( PIXMAPSDIR "next.png" , PIXMAPSDIR "next_o.png", PIXMAPSDIR "next_p.png"),
    compactPlayImgBtn_( PIXMAPSDIR "c_play.png" , PIXMAPSDIR "c_play_o.png", PIXMAPSDIR "c_play_p.png"),
    compactPauseImgBtn_( PIXMAPSDIR "c_pause.png" , PIXMAPSDIR "c_pause_o.png", PIXMAPSDIR "c_pause_p.png"),
    compactStopImgBtn_( PIXMAPSDIR "c_stop.png" , PIXMAPSDIR "c_stop_o.png", PIXMAPSDIR "c_stop_p.png"),
    compactPrevImgBtn_( PIXMAPSDIR "c_prev.png" , PIXMAPSDIR "c_prev_o.png", PIXMAPSDIR "c_prev_p.png"),
    compactNextImgBtn_( PIXMAPSDIR "c_next.png" , PIXMAPSDIR "c_next_o.png", PIXMAPSDIR "c_next_p.png"),
    scrolledWindow_(NULL),
    // load store value?
    prevWidth_(600),
    prevHeight_(371),
    defaultWidth_(600),
    defaultHeight_(371),
    defaultCompactWidth_(345),
    defaultCompactHeight_(82),
    execThread_(NULL),
    inThread_(false), 
    inPlay_(false),
    breakPlay_(false),
    breakPlayForSetDriver_(false),
    randomPlay_(false),
    repeatPlay_(false),
    skipTimeUpdate_(false),
    playQueue_(false),
    listState_(LISTSTATE_EMPTY)
{
    // Do you have mpeg 1 layer 1 file? Anyway, i call it mp1.
    playerFactory_->Register("mp1", &mainManager_, &CreateMP3Player); 
    playerFactory_->Register("mp2", &mainManager_, &CreateMP3Player); 
    playerFactory_->Register("mp3", &mainManager_, &CreateMP3Player); 
    playerFactory_->Register("ogg", &mainManager_, &CreateOGGPlayer); 
    playerFactory_->Register("wma", &mainManager_, &CreateWMAPlayer); 

#ifdef ENABLE_FAAD2
    playerFactory_->Register("mp4", &mainManager_, &CreateAACPlayer);
    playerFactory_->Register("m4a", &mainManager_, &CreateAACPlayer);
    playerFactory_->Register("aac", &mainManager_, &CreateAACPlayer);
#endif 
    
    set_title("Sagua");
    set_resizable(true);
    set_size_request(defaultWidth_, defaultHeight_);
    set_default_icon_from_file( PIXMAPSDIR "sagua.png" );

    add(mainBox_);
    Gtk::Menu_Helpers::MenuList* menuList = &fileMenu_.items();
    menuList->push_back( Gtk::Menu_Helpers::MenuElem(gettext("_Add files"),
		sigc::mem_fun(*this, &MainWnd::OnMenuFileAdd)) );
    menuList->push_back( Gtk::Menu_Helpers::MenuElem(gettext("Add a _directory"),
		sigc::mem_fun(*this, &MainWnd::OnMenuFileDirAdd)) );
    menuList->push_back( Gtk::Menu_Helpers::SeparatorElem());
    menuList->push_back( Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::QUIT,
		sigc::mem_fun(*this, &MainWnd::OnMenuQuit)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_File"), fileMenu_) );

    menuList = &editMenu_.items();
    repeatPlayMenu_.signal_toggled().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuRepeat) );
    randomPlayMenu_.signal_toggled().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuRandom) );
    menuList->push_back( repeatPlayMenu_ );
    menuList->push_back( randomPlayMenu_ );
    menuList->push_back( Gtk::Menu_Helpers::SeparatorElem());
    menuList->push_back( Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::PREFERENCES,
		sigc::mem_fun(*this, &MainWnd::OnMenuPreference)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_Edit"), editMenu_) );

    menuList = &viewMenu_.items();
    mainlistPlayMenu_.signal_activate().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuViewMainList) );
    menuList->push_back(mainlistPlayMenu_);
    dirPlayMenu_.signal_activate().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuViewDir) );
    menuList->push_back(dirPlayMenu_);
    menuList->push_back( Gtk::Menu_Helpers::SeparatorElem());
    compactPlayMenu_.signal_toggled().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuCompactView) );
    menuList->push_back(compactPlayMenu_); 
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_View"), viewMenu_) );

    menuList = &helpMenu_.items();
    menuList->push_back( Gtk::Menu_Helpers::MenuElem(gettext("_About"),
		sigc::mem_fun(mainManager_, &MainManager::About)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_Help"), helpMenu_) );

    mainBox_.pack_start(menuBar_, Gtk::PACK_SHRINK);

    titleDisplayAlignment_.set_padding(12, 12, 0, 0);
    titleDisplayAlignment_.set_size_request(-1, 52);
    titleDisplayAlignment_.add(displayLabel_);
    titleDisplayEventBox_.add(titleDisplayAlignment_);
    titleDisplayEventBox_.signal_button_press_event().connect(
	sigc::mem_fun(*this, &MainWnd::OnButtonPressInDisplayLabel) );
    playerBox_.pack_start(titleDisplayEventBox_);
    timeDisplayBox_.pack_start(*manage(new Gtk::Label));
    timeDisplayBox_.pack_start(timeLabel_, Gtk::PACK_SHRINK);
    timeDisplayBox_.pack_start(totalTimeLabel_, Gtk::PACK_SHRINK);
    timeDisplayBox_.pack_start(*manage(new Gtk::Label));
    playerBox_.pack_start(timeDisplayBox_);
    seekScale_.set_draw_value(false);
    seekScale_.set_update_policy(Gtk::UPDATE_DELAYED);
    seekScale_.signal_adjust_bounds().connect(
	sigc::mem_fun(*this, &MainWnd::OnScaleSeek) );
    seekScaleAlignment_.add(seekScale_);
    playerBox_.pack_start(seekScaleAlignment_);

    playCtlBox_.pack_start(*manage(new Gtk::Label));
    prevImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnPrev) );
    playCtlBox_.pack_start(prevImgBtn_, Gtk::PACK_SHRINK);
    stopImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnStop) );
    playCtlBox_.pack_start(stopImgBtn_, Gtk::PACK_SHRINK);
    pauseImgBtn_.SignalPress().connect(
	    sigc::mem_fun(mainManager_, &MainManager::AudioPauseToggle) );
    playCtlBox_.pack_start(pauseImgBtn_, Gtk::PACK_SHRINK);
    playImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnPlay) );
    playCtlBox_.pack_start(playImgBtn_, Gtk::PACK_SHRINK);
    nextImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnNext) );
    playCtlBox_.pack_start(nextImgBtn_, Gtk::PACK_SHRINK);
    playCtlBox_.pack_start(*manage(new Gtk::Label));
    playerBox_.pack_start(playCtlBox_);

    compactPlayCtlHBox_.pack_start(*manage(new Gtk::HBox));
    compactPrevImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnPrev) );
    compactPlayCtlHBox_.pack_start(compactPrevImgBtn_, Gtk::PACK_SHRINK);
    compactStopImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnStop) );
    compactPlayCtlHBox_.pack_start(compactStopImgBtn_, Gtk::PACK_SHRINK);
    compactPauseImgBtn_.SignalPress().connect(
	    sigc::mem_fun(mainManager_, &MainManager::AudioPauseToggle) );
    compactPlayCtlHBox_.pack_start(compactPauseImgBtn_, Gtk::PACK_SHRINK);
    compactPlayImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnPlay) );
    compactPlayCtlHBox_.pack_start(compactPlayImgBtn_, Gtk::PACK_SHRINK);
    compactNextImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnNext) );
    compactPlayCtlHBox_.pack_start(compactNextImgBtn_, Gtk::PACK_SHRINK);
    compactPlayCtlHBox_.pack_start(*manage(new Gtk::HBox));
    compactPlayCtlBox_.set_spacing(12);
    compactPlayCtlBox_.pack_start(*manage(new Gtk::HBox));
    compactPlayCtlBox_.pack_start(compactPlayCtlHBox_);
    playerBox_.pack_start(compactPlayCtlBox_);

    playerBoxWithVolumn_.pack_start(playerBox_);
    volumnScale_.set_inverted(true);
    volumnScale_.set_draw_value(false);
    volumnScale_.set_value(mainManager_.MixerGetVolumn());
    volumnScale_.signal_adjust_bounds().connect(
	sigc::mem_fun(mainManager_, &MainManager::MixerSetVolumn) );
    volumnAlignment_.add(volumnScale_);
    playerBoxWithVolumn_.pack_start(volumnAlignment_, Gtk::PACK_SHRINK);

    mainBox_.pack_start(playerBoxWithVolumn_, Gtk::PACK_SHRINK);

    titleDisplayEventBoxDefaultStyle_ = titleDisplayEventBox_.get_modifier_style();
    mainWndDefaultStyle_ = get_modifier_style();
 
    show_all_children();

    compactPlayCtlBox_.hide();

    if(arguments.empty())
	LoadLastState(false);
    else {
	LoadLastState(true);

	playlist_->AddFiles(arguments, true);

	playlist_->ForceUpdateToppage();

	playQueue_ = true;
    }

    DisplayVisualPanel(false);

}

MainWnd::~MainWnd()
{
    if(inThread_) {
	mainManager_.AudioUnpause();
	inPlay_ = false;
	player_->Stop();
	execThread_->join();
    }
}

/// @brief Finish play.
///
/// Join thread and after play control.
/// This function called by player when player finish play.
void MainWnd::FinishPlay()
{
    execThread_->join();
    inThread_ = false;

    seekScale_.set_value(0.0);

    if(breakPlayForSetDriver_) {
	breakPlayForSetDriver_ = false;
	mainManager_.AudioSetDriverFromPreference();
    }

    if(!inPlay_) {    
	DisplayVisualPanel(false);

    } else {

	if(breakPlay_) {
	    breakPlay_ = false;
	    OnBtnPlay();
	    
	} else {
	    OnBtnNext();
    
	    if(listState_ == LISTSTATE_LAST) {
		inPlay_ = false;
		DisplayVisualPanel(false);
	    }
	}
    }
}

/// @brief Time display update.
///
/// Get current time from player and display it.
/// This function called by player when player's timer change in 
/// second domain.
void MainWnd::TimeUpdate()
{
    if(skipTimeUpdate_ || !inThread_)
	return;
	
    unsigned long int seconds = player_->CurrentTime();

    timeLabel_.set_label(ConvertSecondToTimeString(seconds));
    seekScale_.set_value(seconds);
}

/// @brief Callback function used when window is received delete 
/// event(window close in WM or else ...).
bool MainWnd::on_delete_event(GdkEventAny* ev)
{
    OnMenuQuit();

    return false;
}

/// @brief Callback function used when window state is changed.
bool MainWnd::on_window_state_event(GdkEventWindowState* ev)
{
    if(ev->new_window_state & GDK_WINDOW_STATE_ICONIFIED) {

	mainManager_.MainWndIconify();
	
    } else {

	Gtk::Widget::on_window_state_event(ev); 	
    
    }

    return false;
}

/// @brief Callback function used when file add menu is clicked.
///
/// Show FileChooserDialog and receive selected files.
void MainWnd::OnMenuFileAdd()
{
    Gtk::FileChooserDialog fileChooserDlg(*this, gettext("Choose files to add"));

    fileChooserDlg.set_local_only(true);
    fileChooserDlg.set_select_multiple(true);

    Gtk::FileFilter filterAll;
    filterAll.set_name(gettext("All files"));
    filterAll.add_pattern("*");
    fileChooserDlg.add_filter(filterAll);

    Gtk::FileFilter filterPlayable;
    filterPlayable.set_name(gettext("All playable files"));
    filterPlayable.add_pattern("*.[Mm][Pp]1");
    filterPlayable.add_pattern("*.[Mm][Pp]2");
    filterPlayable.add_pattern("*.[Mm][Pp]3");
    filterPlayable.add_pattern("*.[Oo][Gg][Gg]");
    filterPlayable.add_pattern("*.[Ww][Mm][Aa]");
    fileChooserDlg.add_filter(filterPlayable);
    
    Gtk::FileFilter filterMpeg;
    filterMpeg.set_name("Mpeg 1 Audio");
    filterMpeg.add_pattern("*.[Mm][Pp]1");
    filterMpeg.add_pattern("*.[Mm][Pp]2");
    filterMpeg.add_pattern("*.[Mm][Pp]3");
    fileChooserDlg.add_filter(filterMpeg);

    Gtk::FileFilter filterOgg;
    filterOgg.set_name("Ogg vorbis");
    filterOgg.add_pattern("*.[Oo][Gg][Gg]");
    fileChooserDlg.add_filter(filterOgg);

    Gtk::FileFilter filterWma;
    filterWma.set_name("Window Media Audio");
    filterWma.add_pattern("*.[Ww][Mm][Aa]");
    fileChooserDlg.add_filter(filterWma);

    Gtk::FileFilter filterList;
    filterList.set_name(gettext("Playlist file"));
    filterList.add_pattern("*.[Mm]3[Uu]");
    fileChooserDlg.add_filter(filterList);

    fileChooserDlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    fileChooserDlg.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

    int result = fileChooserDlg.run();
    fileChooserDlg.hide();

    switch(result)
    {
	case(Gtk::RESPONSE_OK):
	    {
		std::vector<std::string>().swap(selectedFilesVector_);
		
		selectedFilesVector_ = fileChooserDlg.get_filenames();

		playlist_->AddFiles(selectedFilesVector_, true);

		playlist_->ForceUpdatePage();

		break;
	    }
	case(Gtk::RESPONSE_CANCEL):
	default:
	    {

		break;
	    }	
    }
}

/// @brief Callback function used when directory add menu is clicked.
///
/// Show DirBrowseDlg and receive selected directory. Get filenames from
/// selected directory recursively, add them to playlist.
void MainWnd::OnMenuFileDirAdd()
{
    DirBrowseDlg dirBrowseDlg;

    dirBrowseDlg.set_transient_for(*this); 

    int result = dirBrowseDlg.run();
    dirBrowseDlg.hide();

    switch(result)
    {
	case(Gtk::RESPONSE_OK):
	    {
		if(!is_realized())
		    realize();
		
		get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
		while(Glib::MainContext::get_default()->iteration(false));

		playlist_->AddDirectory(dirBrowseDlg.SelectedDirectory(), true);

		get_window()->set_cursor();

		playlist_->ForceUpdatePage();

		break;
	    }
	case(Gtk::RESPONSE_CANCEL):
	default:
	    {

		break;
	    }	
    }
}

/// @brief Callback function used when quit menu is clicked.
///
/// Stop play thread and call MainManager::Quit.
void MainWnd::OnMenuQuit()
{
    OnBtnStop();

    mainManager_.Quit();
}

/// @brief Callback function used when preference menu is clicked.
///
/// Show PrefWnd.
void MainWnd::OnMenuPreference()
{
    PrefWnd prefWnd(mainManager_);
    prefWnd.set_transient_for(*this);
    Gtk::Main::run(prefWnd);
}

/// @brief Callback function used when compact view menu is clicked.
///
/// When compact mode is activated, hide time display, seek bar, 
/// volumn bar, playlist and resize to small.
void MainWnd::OnMenuCompactView()
{
    if(compactPlayMenu_.get_active()) {
	SetBackgroundPixmapInCompactMode();
	
	SetSkipTimeUpdate(true);
	get_size(prevWidth_, prevHeight_);
	playCtlBox_.hide();
	compactPlayCtlBox_.show();
	menuBar_.hide();
	timeDisplayBox_.hide();
	seekScaleAlignment_.hide();	
	volumnAlignment_.hide();
	viewBox_->hide();
	set_size_request(defaultCompactWidth_, defaultCompactHeight_);
	resize(defaultCompactWidth_, defaultCompactHeight_);
	set_resizable(false);
	DisplayTitle(lastTitle_);
    } else {
	UnsetBackgroundPixmapInCompactMode();
	
	SetSkipTimeUpdate(false);
	if(inThread_)
	    TimeUpdate();
	
	compactPlayCtlBox_.hide();
	playCtlBox_.show();
	timeDisplayBox_.show();
	seekScaleAlignment_.show();	
	volumnAlignment_.show();
	viewBox_->show();
	menuBar_.show();
	set_size_request(defaultWidth_, defaultHeight_);
	resize(prevWidth_, prevHeight_);
	set_resizable(true);
	DisplayTitle(lastTitle_);
    }
}

/// @brief Callback function used when mainlist view menu is clicked.
///
/// Remove previous playlist and push Playlist.
void MainWnd::OnMenuViewMainList()
{
    Tracer::Trace trace("MainWnd::OnMenuViewMainList");
       
    if(mainlistPlayMenu_.get_active()) {
	OnBtnStop();
	selectedFile_ = "";
	DisplayTitle("", true);

	if(viewBox_.get() != NULL)
	    mainBox_.remove(*viewBox_);

	viewBox_ = std::auto_ptr<Gtk::HBox>(new Gtk::HBox);

	scrolledWindow_ = manage(new Gtk::ScrolledWindow);
	
	int titleWidth, dummyHeight;
	if(compactPlayMenu_.get_active()) {
	    titleWidth = prevWidth_;
	    titleWidth -= 110;
	} else {
	    get_size(titleWidth, dummyHeight);
	    titleWidth -= 110;
	} 
	    
	playlist_ = manage(new Playlist(*this, "main", titleWidth));
	scrolledWindow_->add(*playlist_);
	scrolledWindow_->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	viewBox_->pack_start(*scrolledWindow_);

	mainBox_.pack_start(*viewBox_);
	viewBox_->show();
	viewBox_->show_all_children();
    }
}

/// @brief Callback function used when directory play view menu is clicked.
///
/// Remove previous playlist and push DirPlayWnd.
void MainWnd::OnMenuViewDir()
{
    Tracer::Trace trace("MainWnd::OnMenuViewDir");
    
    if(dirPlayMenu_.get_active()) {
	
	OnBtnStop();
	selectedFile_ = "";
	DisplayTitle("", true);

	if(viewBox_.get() != NULL)
	    mainBox_.remove(*viewBox_);

	viewBox_ = std::auto_ptr<Gtk::HBox>(new Gtk::HBox);

	int titleWidth, dummyHeight;
	if(compactPlayMenu_.get_active()) {
	    titleWidth = prevWidth_;
	    titleWidth -= 339;
	} else {
	    get_size(titleWidth, dummyHeight);
	    titleWidth -= 339;
	} 

	DirPlayWnd* dirPlayWnd = manage(new DirPlayWnd(*this, titleWidth));
	playlist_ = dirPlayWnd->GetPlaylist();
	viewBox_->pack_start(*dirPlayWnd);

	mainBox_.pack_start(*viewBox_);
	viewBox_->show();
	viewBox_->show_all_children();
    }
}

/// @brief Filname is playable or not.
bool MainWnd::IsPlayable(const std::string& filename)
{
    return playerFactory_->Exist(filename);
}

/// @brief Callback function used when play button is clicked.
///
/// If another thread is running, stop it and run new play thread. 
void MainWnd::OnBtnPlay()
{
    Tracer::Trace trace("MainWnd::OnBtnPlay");
    
    if(selectedFile_.empty()) {
	selectedFile_ = playlist_->Get();

	if(listState_ == LISTSTATE_EMPTY) {
	    trace.Debug("Can't get any filename from list");
	    inPlay_ = false;
	    return;
	}
    }

    inPlay_ = true;

    if(inThread_) {
	trace.Debug("There exists another play thread");

	mainManager_.AudioUnpause();
	breakPlay_ = true;
	player_->Stop();
	return;
    }

    if(IsReadable(selectedFile_)) {
	player_ = playerFactory_->Create(selectedFile_);

	if(player_.get() == NULL) {
	    trace.Debug("Can't create player");
	    OnBtnNext();
	    return;
	}
    } else {
	trace.Debug("selectedFile_ is not readable");
	OnBtnNext();
	return;
    }

    if(player_->Init(selectedFile_)) {
	trace.Debug("Success initialize player");
	execThread_ = Glib::Thread::create(sigc::mem_fun(*player_, &Player::Start), true);
	inThread_ = true;
	DisplayVisualPanel(true);
    } else {
	trace.Debug("Can't initialize player"); 
	OnBtnNext();
    }
}

/// @brief Callback function used when stop button is clicked.
void MainWnd::OnBtnStop()
{
    if(inThread_) {
	
	mainManager_.AudioUnpause();
	inPlay_ = false;
	player_->Stop();
    }

}

/// @brief Callback function used when prev button is clicked.
void MainWnd::OnBtnPrev()
{
    if(!randomPlay_)
	selectedFile_ = playlist_->GetPrev();
    else
	selectedFile_ = playlist_->GetRandom();	

    if(inPlay_ && listState_ != LISTSTATE_FIRST)
	OnBtnPlay();

}

/// @brief Callback function used when next button is clicked.
void MainWnd::OnBtnNext()
{
    if(!randomPlay_)
	selectedFile_ = playlist_->GetNext();
    else	
	selectedFile_ = playlist_->GetRandom();

    if(inPlay_ && listState_ != LISTSTATE_LAST)
	OnBtnPlay();
}

/// @brief Callback function used when seek bar is changed.
void MainWnd::OnScaleSeek(const double& newvalue)
{
    player_->Seek(static_cast<unsigned long>(newvalue));
}

/// @brief Callback function used when repeat menu is clicked.
void MainWnd::OnMenuRepeat()
{
    if(repeatPlayMenu_.get_active())
	repeatPlay_ = true;
    else
	repeatPlay_ = false;		
}

/// @brief Callback function used when random menu is clicked.
void MainWnd::OnMenuRandom()
{
    if(randomPlayMenu_.get_active())
	randomPlay_ = true;
    else
	randomPlay_ = false;		
}

/// @brief Callback function used when display label is pressed.
///
/// When display label is double clicked, toggle compace mode.
bool MainWnd::OnButtonPressInDisplayLabel(GdkEventButton* ev)
{
    if(ev->type == GDK_2BUTTON_PRESS) {
	if(compactPlayMenu_.get_active()) {
	    
	    compactPlayMenu_.set_active(false);
	    
	} else {
	    
	    compactPlayMenu_.set_active(true);
	    
	}
	
	return true;
    }

    return false;
}    

/// @brief Set background pixmap when enter compact mode.
void MainWnd::SetBackgroundPixmapInCompactMode()
{
    if(IsReadable(mainManager_.GetPreference().GeneralCompactBgPixmap())) {
	titleDisplayEventBox_.modify_style(titleDisplayEventBoxDefaultStyle_->copy());
	titleDisplayEventBox_.modify_bg_pixmap(Gtk::STATE_NORMAL, mainManager_.GetPreference().GeneralCompactBgPixmap());
	modify_style(mainWndDefaultStyle_->copy());
	modify_bg_pixmap(Gtk::STATE_NORMAL, mainManager_.GetPreference().GeneralCompactBgPixmap());
    }
}

/// @brief Unset background pixmap when return to normal mode.
void MainWnd::UnsetBackgroundPixmapInCompactMode()
{
    titleDisplayEventBox_.modify_style(titleDisplayEventBoxDefaultStyle_->copy());
    modify_style(mainWndDefaultStyle_->copy());
}

/// @brief Load empty mainlist. 
///
/// This function used when program has executed with command line arguments.
void MainWnd::LoadEmptyMainList()
{
    viewBox_ = std::auto_ptr<Gtk::HBox>(new Gtk::HBox);

    scrolledWindow_ = manage(new Gtk::ScrolledWindow);

    int titleWidth, dummyHeight;
    if(compactPlayMenu_.get_active()) {
	titleWidth = prevWidth_;
	titleWidth -= 110;
    } else {
	get_size(titleWidth, dummyHeight);
	titleWidth -= 110;
    } 
    
    playlist_ = manage(new Playlist(*this, "main", titleWidth, false));
    scrolledWindow_->add(*playlist_);
    scrolledWindow_->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    viewBox_->pack_start(*scrolledWindow_);

    mainBox_.pack_start(*viewBox_);
    viewBox_->show();
    viewBox_->show_all_children();
}

/// @brief Display string.
/// @param title String to display.
/// @param clear Just clear display or not.
void MainWnd::DisplayTitle(const Glib::ustring& title, const bool& clear)
{
    if(!clear) {
	
	lastTitle_ = title;
	Glib::ustring display = "<span weight=\"bold\"";
	
	if(compactPlayMenu_.get_active()) 
	    display += ">";
	else    
	    display += " size=\"large\">";

	// title may be 32 character at maximum, and total length will 
	// be 72 at maximum. And after replace escape sequence, some more
	// length. So who does know exact length? 
	display.reserve(80);
	display += Glib::Markup::escape_text(title);
	display += "</span>";

	displayLabel_.set_label(display);
	displayLabel_.set_use_markup(true);
	displayLabel_.set_alignment(0.5, 0.5);

	set_title("Sagua - " + title);
	mainManager_.TrayIconSetTooltips(title);
	
    } else {
	
	lastTitle_ = "";
	displayLabel_.set_label("");
	set_title("Sagua");
	mainManager_.TrayIconSetTooltips("Sagua");

    }
}

/// @brief Display total time part.
void MainWnd::DisplayTotalTime(const Glib::ustring& time)
{
    totalTimeLabel_.set_label(" / " + time);
}

/// @brief Set maxium range of seek bar.
void MainWnd::SetSeekScaleRange(const unsigned int& max)
{
    seekScale_.set_range(0.0, max);
}

/// @brief Read last state file and restore that state.
///
/// If program has executed with arguments, skip to store view state.
void MainWnd::LoadLastState(const bool& hasArguments)
{
    std::string lastStateFilename = Glib::get_home_dir();
    lastStateFilename += "/.sagua/laststate";

    std::map<std::string, std::string> configMap;
    // for directory entry, load configfile not lowercase!
    if(ConfigfileToMap(lastStateFilename, configMap, false, false)) {
	std::map<std::string, std::string>::const_iterator i, end;
	end = configMap.end();

	i = configMap.find("PlayRepeat");
	if(i != end) {
	    long int repeat = strtol( (i->second).c_str(), NULL, 10);
	    switch(repeat) {
		case 1:
		    repeatPlayMenu_.set_active(true);
		    break;
		    
		case 0:
		default:
		    repeatPlayMenu_.set_active(false);
	    }
	}

	i = configMap.find("PlayRandom");
	if(i != end) {
	    long int random = strtol( (i->second).c_str(), NULL, 10 );
	    switch(random) {
		case 1:
		    randomPlayMenu_.set_active(true);
		    break;
		        
		case 0:
		default:
		    randomPlayMenu_.set_active(false);
	    }
	}
	
	i = configMap.find("LastBrowseDirectory");
	if(i != end)
	    lastBrowsedDirectory_ = i->second;
	else
	    lastBrowsedDirectory_ = Glib::get_home_dir();  

	long int width = 0;
	i = configMap.find("WindowWidth");
	if(i != end)
	    width = strtol( (i->second).c_str(), NULL, 10);

	long int height = 0;
	i = configMap.find("WindowHeight");
	if(i != end)
	    height = strtol( (i->second).c_str(), NULL, 10);
	    
	if(width != 0 && height != 0) {
	    if(width > defaultWidth_ && height > defaultHeight_) {
		prevWidth_ = width;
		prevHeight_ = height;

		resize(prevWidth_, prevHeight_);
	    }
	}

	if(hasArguments)
	    LoadEmptyMainList();
	else {
	    i = configMap.find("View");
	    if(i != end) {
		long int view = strtol( (i->second).c_str(), NULL, 10 );
		switch(view) {
		    case 0:
			mainlistPlayMenu_.activate();
			break;
		    
		    case -1:
			dirPlayMenu_.activate();
			break;
		    
		    default:
			mainlistPlayMenu_.activate();
		}
	    }
	}
	
	i = configMap.find("ViewCompact");
	if(i != end) {
	    long int compact = strtol( (i->second).c_str(), NULL, 10 );
	    switch(compact) {
		case 1:
		    compactPlayMenu_.set_active(true);
		    break;
		    
		case 0:
		default:
		    compactPlayMenu_.set_active(false);
	    }
	}
    } else {
	// else can't load config file to map	
	repeatPlayMenu_.set_active(false);
	randomPlayMenu_.set_active(false);
	lastBrowsedDirectory_ = Glib::get_home_dir();  
	mainlistPlayMenu_.activate();
	compactPlayMenu_.set_active(false);
    }
}

/// @brief Show current play file information.
void MainWnd::CurrentFileInfo()
{
    if(playlist_)
	playlist_->ShowCurrentFileInfoDlg();
}

/// @brief Save last state.
void MainWnd::SaveState()
{
    std::string programDir = Glib::get_home_dir();
    programDir += "/.sagua";

    if(!Glib::file_test(programDir, Glib::FILE_TEST_EXISTS)) {	
	// make dir with permission 755
	if(!mkdir(programDir.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH))
	    return;

    } else if(!Glib::file_test(programDir, Glib::FILE_TEST_IS_DIR)) 
	return;

    std::string lastStateFilename = programDir;
    lastStateFilename += "/laststate";

    std::ofstream lastStateFile(lastStateFilename.c_str());

    if(lastStateFile) {
	int lastWidth = 0;
	int lastHeight = 0;
	if(prevWidth_ != defaultWidth_ || prevHeight_ != defaultHeight_) {
	    lastWidth = prevWidth_;
	    lastHeight = prevHeight_;
	} 

	if(!compactPlayMenu_.get_active()) 
	    get_size(lastWidth, lastHeight);

	int lastViewWindow = 0;	    
	if(dirPlayMenu_.get_active())
	    lastViewWindow = -1;

	
	lastStateFile << "WindowWidth = " << lastWidth << std::endl;
	lastStateFile << "WindowHeight = " << lastHeight << std::endl;
	lastStateFile << "PlayRepeat = " << repeatPlayMenu_.get_active() << std::endl;
	lastStateFile << "PlayRandom = " << randomPlayMenu_.get_active() << std::endl;
	lastStateFile << "View = " << lastViewWindow << std::endl;
	lastStateFile << "ViewCompact = " << compactPlayMenu_.get_active() << std::endl;
	lastStateFile << "LastBrowseDirectory = " << lastBrowsedDirectory_ << std::endl;
    }
}

/// @brief Refresh mixer widget to current PCM volumn.
void MainWnd::RefreshMixer()
{
    volumnScale_.set_value(mainManager_.MixerGetVolumn());
}

/// @brief Change mixer widget to increased value and set volumn to that value.
void MainWnd::VolumnIncrease()
{
    double volumn = volumnScale_.get_value();

    volumn += 5.0;
    if(volumn > 100.0)
	volumn = 100.0;
	
    volumnScale_.set_value(volumn);    
    // Because volumnScale_ connected only signal_adjust_bounds, 
    // refresh mixer by hands ...
    // Need connect signal_change_value or simillar?
    mainManager_.MixerSetVolumn(volumn);
}

/// @brief Change mixer widget to decreased value and set volumn to that value.
void MainWnd::VolumnDecrease()
{
    double volumn = volumnScale_.get_value();

    volumn -= 5.0;
    if(volumn < 0.0)
	volumn = 0.0;

    volumnScale_.set_value(volumn);	
    mainManager_.MixerSetVolumn(volumn);
}

/// @brief Push received filelist to playlist.
///
/// Remove all in current playlist and push received filelist to playlist.
/// And play start.
/// @sa MainManager::SocketReceived
void MainWnd::PushNewListFromSocket(const std::vector<std::string>& list)
{
    playlist_->RemoveAll();

    playlist_->AddFiles(list, true);

    playlist_->ForceUpdateToppage();
    
    OnBtnPlay();

}

/// @brief Change background pixmap used when compact mode.
void MainWnd::ChangeCompactBgPixmap()
{
    if(compactPlayMenu_.get_active()) 
	SetBackgroundPixmapInCompactMode();
}

