// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file prefwin.cc
/// @brief Class for user preference window source file.

#include <iostream>
#include <fstream>

#include <string>

#include <sys/types.h>
#include <sys/stat.h>

#include <libintl.h>

#include <gtkmm/image.h>
#include <gtkmm/box.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/notebook.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/table.h>
#include <gtkmm/entry.h>
#include <gtkmm/radiobuttongroup.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/scale.h>
#include <gtkmm/label.h>
#include <gtkmm/stock.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/menu.h>
#include <gtkmm/optionmenu.h>
#include <gtkmm/combo.h>

#include "../include/fileentry.h"

#include "../include/mainmanager.h"
#include "../include/preference.h"
#include "../include/prefwin.h"
#include "../include/misc.h"
#include "../include/trace.h"

/// @brief Implementaion of PrefWnd.
class PrefWnd::PrefWndImpl
{
    public:
	PrefWndImpl(MainManager& mainManager, PrefWnd& prefWnd);
	~PrefWndImpl() {}

	MainManager& mainManager_;
	PrefWnd& prefWnd_;

	Gtk::Image audioDriverWarningIcon_;

	Gtk::VBox mainBox_;
	FileEntry generalCompactBgPixmap_;
	Gtk::HBox stereoenhanceBox_;
	Gtk::HBox roomBox_;
	Gtk::HBox room2Box_;
	Gtk::HBox harmonyBox_;
	Gtk::HBox rhythmeBox_;
	Gtk::HBox distortionBox_;
	Gtk::HBox EQBox50_, EQBox100_, EQBox200_, EQBox400_, EQBox800_, EQBox1500_;
	Gtk::HBox EQBox3k_, EQBox5k_, EQBox7k_, EQBox10k_, EQBox15k_;
	Gtk::HBox audioAlsa5CardOptionBox_, audioAlsa5DevOptionBox_;
	Gtk::HBox audioAlsa9OptionBox_; 
	Gtk::HBox audioOssOptionBox_;
	Gtk::HBox audioSunOptionBox_, audioAixsOptionBox_;
	Gtk::HBox audioDriverWarningBox_;
	Gtk::Notebook mainNotebook_;
	Gtk::HButtonBox mainBtnBox_;

	Gtk::ScrolledWindow driverScrolledWindow_;

	Gtk::Table generalTable_;
	Gtk::Table driverTable_;
	Gtk::Table effectTable_;
	Gtk::Table graphicEQTable_;

	Gtk::Menu generalTrayIconActionMenu_;
	Gtk::OptionMenu generalTrayIconActionOptionMenu_;

	Gtk::Entry audioAlsa5Card_, audioAlsa5Dev_;
	Gtk::Entry audioAlsa9Dev_;
	Gtk::Entry audioOssDsp_;
	Gtk::Entry audioSunDev_;
	Gtk::Entry audioAixsDev_;
	Gtk::Entry generalMixerDev_;
	Gtk::Entry generalAssumeCharset_;

	Gtk::RadioButtonGroup driverBtnGroup_;
	Gtk::RadioButton audioDefaultBtn_;
	Gtk::RadioButton audioAlsa5Btn_;
	Gtk::RadioButton audioAlsa9Btn_;
	Gtk::RadioButton audioEsdBtn_;
	Gtk::RadioButton audioIrixBtn_;
	Gtk::RadioButton audioOssBtn_;
	Gtk::RadioButton audioSunBtn_;
	Gtk::RadioButton audioAixsBtn_;

	Gtk::CheckButton generalLeaveOnTrayWhenIconifyBtn_;
	Gtk::CheckButton bassboostBtn_;
	Gtk::CheckButton bassboost2Btn_;
	Gtk::CheckButton trebleboostBtn_;
	Gtk::CheckButton stereoenhanceBtn_; 
	Gtk::CheckButton rhythmeBtn_;
	Gtk::CheckButton roomBtn_;
	Gtk::CheckButton roomBtn2_;
	Gtk::CheckButton harmonyBtn_;
	Gtk::CheckButton distortionBtn_;
	Gtk::CheckButton graphicEQBtn_;

	Gtk::HScale stereoenhanceScale_;
	Gtk::HScale roomScale_;
	Gtk::HScale room2Scale_;
	Gtk::HScale harmonyScale_;
	Gtk::HScale rhythmeScale_;
	Gtk::HScale distortionScale_;
	Gtk::HScale EQ50_, EQ100_, EQ200_, EQ400_, EQ800_, EQ1500_;
	Gtk::HScale EQ3k_, EQ5k_, EQ7k_, EQ10k_, EQ15k_;

	Gtk::Button audioDriverTestBtn_;
	Gtk::Button cancelBtn_, okBtn_;

	Gtk::Label generalLabel_;
	Gtk::Label generalTrayIconActionLabel_;
	Gtk::Label generalAssumeCharsetLabel_;
	Gtk::Label generalCompactBgPixmapLabel_;
	Gtk::Label generalMixerDevLabel_;
	Gtk::Label audioDriverLabel_;
	Gtk::Label audioAlsa5CardLabel_;
	Gtk::Label audioAlsa5DevLabel_;
	Gtk::Label audioAlsa9DevLabel_;
	Gtk::Label audioOssDspLabel_;
	Gtk::Label audioSunDevLabel_;
	Gtk::Label audioAixsDevLabel_;
	Gtk::Label audioDriverTestLabel_;
	Gtk::Label audioDriverTestResultLabel_;
	Gtk::Label audioDriverWarningLabel_;
	Gtk::Label stableEffectLabel_;
	Gtk::Label developEffectLabel_; 
	Gtk::Label graphicEQLabel_;

	Gtk::Tooltips generalTrayIconActionTooltips_;
	Gtk::Tooltips generalAssumeCharsetTooltips_;
	Gtk::Tooltips audioAlsa5Tooltips_;
	Gtk::Tooltips audioAlsa9Tooltips_;
	Gtk::Tooltips audioAlsa9DeviceTooltips_;
	Gtk::Tooltips bassBoostTooltips_;
	Gtk::Tooltips bassBoost2Tooltips_;
	Gtk::Tooltips trebleBoostTooltips_;
	Gtk::Tooltips stereoenhanceTooltips_;
	Gtk::Tooltips roomTooltips_;
	Gtk::Tooltips room2Tooltips_;

	std::string selectedTrayIconAction_;
	std::string previousCompactBgPixmap_;
	std::string previousMixerDev_;
	std::string previousAudioDriver_;
	std::string selectedAudioDriver_;
	std::string previousAlsa5Card_;
	std::string previousAlsa5Dev_;
	std::string previousAlsa9Dev_;
	std::string previousOssDsp_;
	std::string previousSunDev_;
	std::string previousAixsDev_;

	void OnMenuTrayIconAction(const std::string& action);

	void OnBtnAudioDriver(const char* driver);
	void OnBtnAudioDriverTest();

	void OnBtnStereoEnhance();
	void OnBtnRoom();
	void OnBtnRoom2();
	void OnBtnHarmony();
	void OnBtnRhythme();
	void OnBtnDistortion();
	void OnBtnGraphicEQ();

	void OnBtnCancel();
	void OnBtnOk();

};

PrefWnd::PrefWnd(MainManager& mainManager)
    : impl_(new PrefWndImpl(mainManager, *this))
{}

PrefWnd::~PrefWnd()
{}

PrefWnd::PrefWndImpl::PrefWndImpl(MainManager& mainManager, PrefWnd& prefWnd)
    : mainManager_(mainManager),
    prefWnd_(prefWnd),
    audioDriverWarningIcon_(Gtk::Stock::DIALOG_WARNING, Gtk::ICON_SIZE_LARGE_TOOLBAR),
    generalCompactBgPixmap_(prefWnd, gettext("Background image file used when main window is compact mode.")),
    mainBtnBox_(Gtk::BUTTONBOX_END, 12),
    generalTable_(3, 19),
    driverTable_(5, 19),
    effectTable_(3, 19), 
    graphicEQTable_(5, 19),
    audioDefaultBtn_(driverBtnGroup_, gettext("Use default driver\n(auto detect)")),
    audioAlsa5Btn_(driverBtnGroup_, "Alsa 0.5.x"),
    audioAlsa9Btn_(driverBtnGroup_, "Alsa 0.9.x"),
    audioEsdBtn_(driverBtnGroup_, "ESD"),
    audioIrixBtn_(driverBtnGroup_, "Irix"),
    audioOssBtn_(driverBtnGroup_, "OSS"),
    audioSunBtn_(driverBtnGroup_, "SUN"),
    audioAixsBtn_(driverBtnGroup_, "Aixs"),
    generalLeaveOnTrayWhenIconifyBtn_(gettext("Leave on tray only when iconify")),
    bassboostBtn_("Bass Boost"),
    bassboost2Btn_("Bass Boost2"),
    trebleboostBtn_("High Frequency Boost"),
    stereoenhanceBtn_("Stereo Enhance"),
    rhythmeBtn_("Rhythme Effect"),
    roomBtn_("Room Effect"),
    roomBtn2_("Room2 Effect"),
    harmonyBtn_("Harmony Effect"),
    distortionBtn_("Distortion Effect"),
    graphicEQBtn_( gettext("Use equalizer") ),
    stereoenhanceScale_(0.1, 2.1, 0.1),
    roomScale_(0.1, 1.1, 0.1),
    room2Scale_(0.1, 1.1, 0.1),
    harmonyScale_(0.1, 1.1, 0.1),
    rhythmeScale_(0.1, 1.6, 0.1),
    distortionScale_(1.0, 10.1, 0.1),
    // Is it efficient more than 6 dB? anyway...
    EQ50_(-12.0, 12.1, 0.1),
    EQ100_(-12.0, 12.1, 0.1),
    EQ200_(-12.0, 12.1, 0.1),
    EQ400_(-12.0, 12.1, 0.1),
    EQ800_(-12.0, 12.1, 0.1),
    EQ1500_(-12.0, 12.1, 0.1),
    EQ3k_(-12.0, 12.1, 0.1),
    EQ5k_(-12.0, 12.1, 0.1),
    EQ7k_(-12.0, 12.1, 0.1),
    EQ10k_(-12.0, 12.1, 0.1),
    EQ15k_(-12.0, 12.1, 0.1),
    audioDriverTestBtn_(gettext("Test")),
    cancelBtn_(Gtk::Stock::CANCEL),
    okBtn_(Gtk::Stock::OK),
    generalLabel_(gettext("<span weight=\"bold\">General</span>"), 0.0, 0.5),
    generalTrayIconActionLabel_(gettext("Tray icon click action : "), 0.0, 0.5),
    generalAssumeCharsetLabel_(gettext("Assume charset : "), 0.0, 0.5),
    generalCompactBgPixmapLabel_(gettext("Compact mode background : "), 0.0, 0.5),
    generalMixerDevLabel_(gettext("Mixer device : "), 0.0, 0.5),
    audioDriverLabel_(gettext("<span weight=\"bold\">Audio Driver</span>"), 0.0, 0.5),
    audioAlsa5CardLabel_("Card : ", 0.0, 0.5),
    audioAlsa5DevLabel_("Device : ", 0.0, 0.5),
    audioAlsa9DevLabel_("Device : ", 0.0, 0.5),
    audioOssDspLabel_(gettext("Path : "), 0.0, 0.5),
    audioSunDevLabel_(gettext("Path : "), 0.0, 0.5),
    audioAixsDevLabel_(gettext("Path : "), 0.0, 0.5),
    audioDriverTestLabel_(gettext("<span weight=\"bold\">Audio Driver Test</span>"), 0.0, 0.5),
    audioDriverWarningLabel_(gettext("Change will stop your playing!")), 
    stableEffectLabel_(gettext("<span weight=\"bold\">Effector</span>"), 0.0, 0.5),
    developEffectLabel_(gettext("<span weight=\"bold\">Experimental Effector</span>"), 0.0, 0.5),
    graphicEQLabel_(gettext("<span weight=\"bold\">Equalizer</span>"), 0.0, 0.5)
{
    prefWnd_.set_title(gettext("Preference"));
    prefWnd_.set_resizable(false);

    mainBox_.set_border_width(12);
    mainBox_.set_spacing(12);
    prefWnd_.add(mainBox_);

    generalTable_.set_border_width(12);
    generalTable_.set_row_spacings(6);
    generalTable_.set_col_spacing(0, 12);
    generalTable_.set_col_spacing(1, 12);

    generalLabel_.set_use_markup(true);
    generalTable_.attach(generalLabel_, 0, 3, 0, 1, Gtk::FILL);

    generalTable_.attach(generalTrayIconActionLabel_, 1, 2, 1, 2, Gtk::FILL);

    Gtk::Menu_Helpers::MenuList& list = generalTrayIconActionMenu_.items();
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Open window"),
		sigc::bind( sigc::mem_fun(*this, &PrefWndImpl::OnMenuTrayIconAction), "openwin") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("File info."),
		sigc::bind( sigc::mem_fun(*this, &PrefWndImpl::OnMenuTrayIconAction), "fileinfo") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Stop"),
		sigc::bind( sigc::mem_fun(*this, &PrefWndImpl::OnMenuTrayIconAction), "stop") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Pause"),
		sigc::bind( sigc::mem_fun(*this, &PrefWndImpl::OnMenuTrayIconAction), "pause") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Play"),
		sigc::bind( sigc::mem_fun(*this, &PrefWndImpl::OnMenuTrayIconAction), "play") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Prev"),
		sigc::bind( sigc::mem_fun(*this, &PrefWndImpl::OnMenuTrayIconAction), "prev") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Next"),
		sigc::bind( sigc::mem_fun(*this, &PrefWndImpl::OnMenuTrayIconAction), "next") ) );

    selectedTrayIconAction_ = mainManager_.GetPreference().GeneralTrayIconAction();
    if(selectedTrayIconAction_ == "openwin")
	generalTrayIconActionMenu_.set_active(0);
    else if(selectedTrayIconAction_ == "fileinfo")
	generalTrayIconActionMenu_.set_active(1);
    else if(selectedTrayIconAction_ == "stop")
	generalTrayIconActionMenu_.set_active(2);
    else if(selectedTrayIconAction_ == "pause")
	generalTrayIconActionMenu_.set_active(3);
    else if(selectedTrayIconAction_ == "play")
	generalTrayIconActionMenu_.set_active(4);			
    else if(selectedTrayIconAction_ == "prev")
	generalTrayIconActionMenu_.set_active(5);
    else if(selectedTrayIconAction_ == "next")
	generalTrayIconActionMenu_.set_active(6);	
    else
	generalTrayIconActionMenu_.set_active(0);

    generalTrayIconActionOptionMenu_.set_menu(generalTrayIconActionMenu_);
    generalTrayIconActionTooltips_.set_tip(generalTrayIconActionOptionMenu_, 
	    gettext("Action when you click a tray icon in first button"));
    generalTable_.attach(generalTrayIconActionOptionMenu_, 2, 3, 1, 2, Gtk::FILL);

    generalLeaveOnTrayWhenIconifyBtn_.set_active(
	    mainManager_.GetPreference().GeneralLeaveOnTrayWhenIconify() );
    generalTable_.attach(generalLeaveOnTrayWhenIconifyBtn_, 1, 3, 2, 3, Gtk::FILL);

    generalAssumeCharset_.set_text(mainManager_.GetPreference().GeneralAssumeCharset());
    if(!Glib::get_charset())
	generalAssumeCharset_.set_sensitive(false);
    generalAssumeCharsetTooltips_.set_tip(generalAssumeCharset_,
	    gettext("Assume charset when convert not valid UTF-8 string to UTF-8 and machine's charset is UTF-8"));	
    generalTable_.attach(generalAssumeCharsetLabel_, 1, 2, 3, 4, Gtk::FILL);
    generalTable_.attach(generalAssumeCharset_, 2, 3, 3, 4, Gtk::FILL); 

    generalTable_.attach(generalCompactBgPixmapLabel_, 1, 2, 4, 5, Gtk::FILL);
    previousCompactBgPixmap_ = mainManager.GetPreference().GeneralCompactBgPixmap();
    generalCompactBgPixmap_.SetEntryText(previousCompactBgPixmap_);
    generalTable_.attach(generalCompactBgPixmap_, 2, 3, 4, 5, Gtk::FILL);

    generalTable_.attach(generalMixerDevLabel_, 1, 2, 5, 6, Gtk::FILL);
    previousMixerDev_ = mainManager_.MixerDev();
    generalMixerDev_.set_text(previousMixerDev_);
    generalTable_.attach(generalMixerDev_, 2, 3, 5, 6, Gtk::FILL);

    generalTable_.attach(*manage(new Gtk::Label), 0, 3, 6, 19, Gtk::FILL);

    mainNotebook_.append_page(generalTable_, gettext("General"));

    driverTable_.set_border_width(12);
    driverTable_.set_row_spacings(6);
    driverTable_.set_col_spacing(0, 12);
    driverTable_.set_col_spacing(1, 12);

    audioDriverLabel_.set_use_markup(true);
    driverTable_.attach(audioDriverLabel_, 0, 5, 0, 1, Gtk::FILL);

    audioDriverWarningBox_.pack_start(audioDriverWarningIcon_, Gtk::PACK_SHRINK);
    audioDriverWarningBox_.pack_start(audioDriverWarningLabel_, Gtk::PACK_SHRINK);
    driverTable_.attach(audioDriverWarningBox_, 1, 5, 1, 2, Gtk::FILL);

    previousAudioDriver_ = mainManager_.GetPreference().AudioDriver();
    selectedAudioDriver_ = previousAudioDriver_;

    audioDefaultBtn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "default") );
    driverTable_.attach(audioDefaultBtn_, 1, 4, 2, 3, Gtk::FILL);
    driverTable_.attach(*manage(new Gtk::Label), 4, 5, 2, 3, Gtk::EXPAND);

    audioAlsa5Btn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "alsa") );
    audioAlsa5Tooltips_.set_tip(audioAlsa5Btn_, gettext("Driver for alsa 0.5.x"));
    driverTable_.attach(audioAlsa5Btn_, 1, 4, 3, 4, Gtk::FILL);

    driverTable_.attach(audioAlsa5CardLabel_, 2, 3, 4, 5, Gtk::FILL);
    audioAlsa5CardOptionBox_.set_spacing(12);
    audioAlsa5CardOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAlsa5Card_ = mainManager_.GetPreference().AudioAlsa5Card();
    audioAlsa5Card_.set_text(previousAlsa5Card_);
    audioAlsa5CardOptionBox_.pack_start(audioAlsa5Card_);
    driverTable_.attach(audioAlsa5CardOptionBox_, 3, 5, 4, 5, Gtk::FILL);

    driverTable_.attach(audioAlsa5DevLabel_, 2, 3, 5, 6, Gtk::FILL);
    audioAlsa5DevOptionBox_.set_spacing(12);
    audioAlsa5DevOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAlsa5Dev_ = mainManager_.GetPreference().AudioAlsa5Dev();
    audioAlsa5Dev_.set_text(previousAlsa5Dev_);
    audioAlsa5DevOptionBox_.pack_start(audioAlsa5Dev_);
    driverTable_.attach(audioAlsa5DevOptionBox_, 3, 5, 5, 6, Gtk::FILL);

    audioAlsa9Btn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "alsa09") );
    audioAlsa9Tooltips_.set_tip(audioAlsa9Btn_, gettext("Driver for alsa 0.9.x and 1.0.x"));
    driverTable_.attach(audioAlsa9Btn_, 1, 4, 6, 7, Gtk::FILL);

    driverTable_.attach(audioAlsa9DevLabel_, 2, 3, 7, 8, Gtk::FILL);
    audioAlsa9OptionBox_.set_spacing(12);
    audioAlsa9OptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAlsa9Dev_ = mainManager_.GetPreference().AudioAlsa9Dev();
    audioAlsa9Dev_.set_text(previousAlsa9Dev_);
    audioAlsa9DeviceTooltips_.set_tip(audioAlsa9Dev_, gettext("ex) hw:0,0 or ens1371(naming in asoundrc)"));
    audioAlsa9OptionBox_.pack_start(audioAlsa9Dev_);
    driverTable_.attach(audioAlsa9OptionBox_, 3, 5, 7, 8, Gtk::FILL);

    audioEsdBtn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "esd") );	
    driverTable_.attach(audioEsdBtn_, 1, 4, 8, 9, Gtk::FILL);

    audioIrixBtn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "irix") );
    driverTable_.attach(audioIrixBtn_, 1, 4, 9, 10, Gtk::FILL);


    audioOssBtn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "oss") );
    driverTable_.attach(audioOssBtn_, 1, 4, 10, 11, Gtk::FILL);

    driverTable_.attach(audioOssDspLabel_, 2, 3, 11, 12, Gtk::FILL);
    audioOssOptionBox_.set_spacing(12);
    audioOssOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousOssDsp_ = mainManager_.GetPreference().AudioOssDsp();
    audioOssDsp_.set_text(previousOssDsp_);
    audioOssOptionBox_.pack_start(audioOssDsp_);
    driverTable_.attach(audioOssOptionBox_, 3, 5, 11, 12, Gtk::FILL);

    audioSunBtn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "sun") );
    driverTable_.attach(audioSunBtn_, 1, 4, 12, 13, Gtk::FILL);

    driverTable_.attach(audioSunDevLabel_, 2, 3, 13, 14, Gtk::FILL);
    audioSunOptionBox_.set_spacing(12);
    audioSunOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousSunDev_ = mainManager_.GetPreference().AudioSunDev();
    audioSunDev_.set_text(previousSunDev_);
    audioSunOptionBox_.pack_start(audioSunDev_);
    driverTable_.attach(audioSunOptionBox_, 3, 5, 13, 14, Gtk::FILL);

    audioAixsBtn_.signal_clicked().connect( sigc::bind(
		sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriver), "aixs") );
    driverTable_.attach(audioAixsBtn_, 1, 4, 14, 15, Gtk::FILL);

    driverTable_.attach(audioAixsDevLabel_, 2, 3, 15, 16, Gtk::FILL);
    audioAixsOptionBox_.set_spacing(12);
    audioAixsOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAixsDev_ = mainManager_.GetPreference().AudioAixsDev();
    audioAixsDev_.set_text(previousAixsDev_);
    audioAixsOptionBox_.pack_start(audioAixsDev_);
    driverTable_.attach(audioAixsOptionBox_, 3, 5, 15, 16, Gtk::FILL);

    audioDriverTestLabel_.set_use_markup(true);
    driverTable_.attach(audioDriverTestLabel_, 0, 5, 17, 18, Gtk::FILL);

    audioDriverTestBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnAudioDriverTest) );
    driverTable_.attach(audioDriverTestBtn_, 1, 3, 18, 19, Gtk::FILL);
    driverTable_.attach(audioDriverTestResultLabel_, 3, 5, 18, 19, Gtk::FILL);

    audioAlsa5CardOptionBox_.set_sensitive(false);
    audioAlsa5DevOptionBox_.set_sensitive(false);
    audioAlsa9OptionBox_.set_sensitive(false);
    audioOssOptionBox_.set_sensitive(false);
    audioSunOptionBox_.set_sensitive(false);
    audioAixsOptionBox_.set_sensitive(false);

    if(selectedAudioDriver_ == "alsa") {
	audioAlsa5Btn_.set_active(true);
	audioAlsa5CardOptionBox_.set_sensitive(true);
	audioAlsa5DevOptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "alsa09") {
	audioAlsa9Btn_.set_active(true);
	audioAlsa9OptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "esd") {
	audioEsdBtn_.set_active(true);
    } else if(selectedAudioDriver_ == "irix") {
	audioIrixBtn_.set_active(true);
    } else if(selectedAudioDriver_ == "oss") {
	audioOssBtn_.set_active(true);
	audioOssOptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "sun") {
	audioSunBtn_.set_active(true);
	audioSunOptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "aixs") {
	audioAixsBtn_.set_active(true);
	audioAixsOptionBox_.set_sensitive(true);
    } else
	audioDefaultBtn_.set_active(true);

    driverScrolledWindow_.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    driverScrolledWindow_.add(driverTable_);

    mainNotebook_.append_page(driverScrolledWindow_, gettext("Audio Driver"));

    effectTable_.set_border_width(12);
    effectTable_.set_row_spacings(6);
    effectTable_.set_col_spacing(0, 12);
    effectTable_.set_col_spacing(1, 12);

    stableEffectLabel_.set_use_markup(true);
    effectTable_.attach(stableEffectLabel_, 0, 3, 0, 1, Gtk::FILL);

    bassboostBtn_.set_active(mainManager_.GetPreference().EfxBassBoost());
    bassBoostTooltips_.set_tip(bassboostBtn_, gettext("Boost low frequency band"));
    effectTable_.attach(bassboostBtn_, 1, 2, 1, 2, Gtk::FILL);
    effectTable_.attach(*manage(new Gtk::Label), 2, 3, 1, 2, Gtk::EXPAND);

    stereoenhanceBtn_.set_active(mainManager_.GetPreference().EfxStereoEnhance());
    stereoenhanceBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnStereoEnhance) );
    stereoenhanceTooltips_.set_tip(stereoenhanceBtn_, gettext("Stereo enhance effect"));
    effectTable_.attach(stereoenhanceBtn_, 1, 2, 2, 3, Gtk::FILL);
    stereoenhanceScale_.set_value_pos(Gtk::POS_RIGHT);
    stereoenhanceScale_.set_draw_value();
    stereoenhanceScale_.set_value(mainManager_.GetPreference().EfxStereoEnhanceAmount());
    stereoenhanceScale_.set_sensitive(mainManager_.GetPreference().EfxStereoEnhance());
    stereoenhanceBox_.set_spacing(12);
    stereoenhanceBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    stereoenhanceBox_.pack_start(stereoenhanceScale_);
    effectTable_.attach(stereoenhanceBox_, 1, 3, 3, 4, Gtk::FILL);
    effectTable_.set_row_spacing(2, 3);

    effectTable_.attach(*manage(new Gtk::Label), 0, 3, 4, 5, Gtk::FILL);

    developEffectLabel_.set_use_markup(true);
    effectTable_.attach(developEffectLabel_, 0, 3, 5, 6, Gtk::FILL);

    bassboost2Btn_.set_active(mainManager_.GetPreference().EfxBassBoost2());
    bassBoost2Tooltips_.set_tip(bassboost2Btn_, gettext("Experimental low frequency band boost"));
    effectTable_.attach(bassboost2Btn_, 1, 2, 6, 7, Gtk::FILL);

    trebleboostBtn_.set_active(mainManager_.GetPreference().EfxTrebleBoost());
    trebleBoostTooltips_.set_tip(trebleboostBtn_, 
	    gettext("Boost high frequency band, also you can here high frequency noise"));
    effectTable_.attach(trebleboostBtn_, 1, 2, 7, 8, Gtk::FILL);

    roomBtn_.set_active(mainManager_.GetPreference().EfxRoom());
    roomBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnRoom) );
    roomTooltips_.set_tip(roomBtn_,
	    gettext("Experimental space effector"));
    effectTable_.attach(roomBtn_, 1, 2, 8, 9, Gtk::FILL);
    roomScale_.set_value_pos(Gtk::POS_RIGHT);
    roomScale_.set_draw_value();
    roomScale_.set_value(mainManager_.GetPreference().EfxRoomAmount());
    roomScale_.set_sensitive(mainManager_.GetPreference().EfxRoom());
    roomBox_.set_spacing(12);
    roomBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    roomBox_.pack_start(roomScale_);
    effectTable_.attach(roomBox_, 1, 3, 9, 10, Gtk::FILL);
    effectTable_.set_row_spacing(8, 3);

    roomBtn2_.set_active(mainManager_.GetPreference().EfxRoom2());
    roomBtn2_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnRoom2) );
    room2Tooltips_.set_tip(roomBtn2_,
	    gettext("Another experimental space effector"));
    effectTable_.attach(roomBtn2_, 1, 2, 10, 11, Gtk::FILL);
    room2Scale_.set_value_pos(Gtk::POS_RIGHT);
    room2Scale_.set_draw_value();
    room2Scale_.set_value(mainManager_.GetPreference().EfxRoom2Amount());
    room2Scale_.set_sensitive(mainManager_.GetPreference().EfxRoom2());
    room2Box_.set_spacing(12);
    room2Box_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    room2Box_.pack_start(room2Scale_);
    effectTable_.attach(room2Box_, 1, 3, 11, 12, Gtk::FILL);
    effectTable_.set_row_spacing(10, 3);


    harmonyBtn_.set_active(mainManager_.GetPreference().EfxHarmony());
    harmonyBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnHarmony) );
    effectTable_.attach(harmonyBtn_, 1, 2, 12, 13, Gtk::FILL);
    harmonyScale_.set_value_pos(Gtk::POS_RIGHT);
    harmonyScale_.set_draw_value();
    harmonyScale_.set_value(mainManager_.GetPreference().EfxHarmonyAmount());
    harmonyScale_.set_sensitive(mainManager_.GetPreference().EfxHarmony());
    harmonyBox_.set_spacing(12);
    harmonyBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    harmonyBox_.pack_start(harmonyScale_);
    effectTable_.attach(harmonyBox_, 1, 3, 13, 14, Gtk::FILL);
    effectTable_.set_row_spacing(12, 3);

    rhythmeBtn_.set_active(mainManager_.GetPreference().EfxRhythme());
    rhythmeBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnRhythme) );
    effectTable_.attach(rhythmeBtn_, 1, 2, 14, 15, Gtk::FILL);
    rhythmeScale_.set_value_pos(Gtk::POS_RIGHT);
    rhythmeScale_.set_draw_value();
    rhythmeScale_.set_value(mainManager_.GetPreference().EfxRhythmeAmount());
    rhythmeScale_.set_sensitive(mainManager_.GetPreference().EfxRhythme());
    rhythmeBox_.set_spacing(12);
    rhythmeBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    rhythmeBox_.pack_start(rhythmeScale_);
    effectTable_.attach(rhythmeBox_, 1, 3, 15, 16, Gtk::FILL);
    effectTable_.set_row_spacing(14, 3);

    distortionBtn_.set_active(mainManager_.GetPreference().EfxDistortion());
    distortionBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnDistortion) );
    effectTable_.attach(distortionBtn_, 1, 2, 16, 17, Gtk::FILL);
    distortionScale_.set_value_pos(Gtk::POS_RIGHT);
    distortionScale_.set_draw_value();
    distortionScale_.set_value(mainManager_.GetPreference().EfxDistortionAmount());
    distortionScale_.set_sensitive(mainManager_.GetPreference().EfxDistortion());
    distortionBox_.set_spacing(12);
    distortionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    distortionBox_.pack_start(distortionScale_);
    effectTable_.attach(distortionBox_, 1, 3, 17, 18, Gtk::FILL);
    effectTable_.set_row_spacing(16, 3);

    effectTable_.attach(*manage(new Gtk::Label), 0, 3, 18, 19, Gtk::FILL);

    mainNotebook_.append_page(effectTable_, gettext("Effector"));

    graphicEQTable_.set_border_width(12);
    graphicEQTable_.set_row_spacings(6);
    graphicEQTable_.set_col_spacing(0, 12);
    graphicEQTable_.set_col_spacing(1, 12);

    graphicEQLabel_.set_use_markup(true);
    graphicEQTable_.attach(graphicEQLabel_, 0, 5, 0, 1, Gtk::FILL);

    graphicEQBtn_.set_active(mainManager_.GetPreference().EfxGraphicEQ());
    graphicEQBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnGraphicEQ) );
    graphicEQTable_.attach(graphicEQBtn_, 1, 4, 1, 2, Gtk::FILL);
    graphicEQTable_.attach(*manage(new Gtk::Label), 4, 5, 1, 2, Gtk::EXPAND);

    EQ50_.set_value_pos(Gtk::POS_RIGHT);
    EQ50_.set_draw_value();
    EQ50_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[0] );
    EQ50_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox50_.set_spacing(12);
    EQBox50_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox50_.pack_start(EQ50_);
    graphicEQTable_.attach(*manage(new Gtk::Label("50 Hz", 0.0, 0.5)), 2, 3, 2, 3, Gtk::FILL);
    graphicEQTable_.attach(EQBox50_, 3, 5, 2, 3, Gtk::FILL);

    EQ100_.set_value_pos(Gtk::POS_RIGHT);
    EQ100_.set_draw_value();
    EQ100_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[1] );
    EQ100_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox100_.set_spacing(12);
    EQBox100_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox100_.pack_start(EQ100_);
    graphicEQTable_.attach(*manage(new Gtk::Label("100 Hz", 0.0, 0.5)), 2, 3, 3, 4, Gtk::FILL);
    graphicEQTable_.attach(EQBox100_, 3, 5, 3, 4, Gtk::FILL);

    EQ200_.set_value_pos(Gtk::POS_RIGHT);
    EQ200_.set_draw_value();
    EQ200_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[2] );
    EQ200_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox200_.set_spacing(12);
    EQBox200_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox200_.pack_start(EQ200_);
    graphicEQTable_.attach(*manage(new Gtk::Label("200 Hz", 0.0, 0.5)), 2, 3, 4, 5, Gtk::FILL);
    graphicEQTable_.attach(EQBox200_, 3, 5, 4, 5, Gtk::FILL);

    EQ400_.set_value_pos(Gtk::POS_RIGHT);
    EQ400_.set_draw_value();
    EQ400_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[3] );
    EQ400_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox400_.set_spacing(12);
    EQBox400_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox400_.pack_start(EQ400_);
    graphicEQTable_.attach(*manage(new Gtk::Label("400 Hz", 0.0, 0.5)), 2, 3, 5, 6, Gtk::FILL);
    graphicEQTable_.attach(EQBox400_, 3, 5, 5, 6, Gtk::FILL);

    EQ800_.set_value_pos(Gtk::POS_RIGHT);
    EQ800_.set_draw_value();
    EQ800_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[4] );
    EQ800_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox800_.set_spacing(12);
    EQBox800_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox800_.pack_start(EQ800_);
    graphicEQTable_.attach(*manage(new Gtk::Label("800 Hz", 0.0, 0.5)), 2, 3, 6, 7, Gtk::FILL);
    graphicEQTable_.attach(EQBox800_, 3, 5, 6, 7, Gtk::FILL);

    EQ1500_.set_value_pos(Gtk::POS_RIGHT);
    EQ1500_.set_draw_value();
    EQ1500_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[5] );
    EQ1500_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox1500_.set_spacing(12);
    EQBox1500_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox1500_.pack_start(EQ1500_);
    graphicEQTable_.attach(*manage(new Gtk::Label("1.5 kHz", 0.0, 0.5)), 2, 3, 7, 8, Gtk::FILL);
    graphicEQTable_.attach(EQBox1500_, 3, 5, 7, 8, Gtk::FILL);

    EQ3k_.set_value_pos(Gtk::POS_RIGHT);
    EQ3k_.set_draw_value();
    EQ3k_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[6] );
    EQ3k_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox3k_.set_spacing(12);
    EQBox3k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox3k_.pack_start(EQ3k_);
    graphicEQTable_.attach(*manage(new Gtk::Label("3 kHz", 0.0, 0.5)), 2, 3, 8, 9, Gtk::FILL);
    graphicEQTable_.attach(EQBox3k_, 3, 5, 8, 9, Gtk::FILL);

    EQ5k_.set_value_pos(Gtk::POS_RIGHT);
    EQ5k_.set_draw_value();
    EQ5k_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[7] );
    EQ5k_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox5k_.set_spacing(12);
    EQBox5k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox5k_.pack_start(EQ5k_);
    graphicEQTable_.attach(*manage(new Gtk::Label("5 kHz", 0.0, 0.5)), 2, 3, 9, 10, Gtk::FILL);
    graphicEQTable_.attach(EQBox5k_, 3, 5, 9, 10, Gtk::FILL);

    EQ7k_.set_value_pos(Gtk::POS_RIGHT);
    EQ7k_.set_draw_value();
    EQ7k_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[8] );
    EQ7k_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox7k_.set_spacing(12);
    EQBox7k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox7k_.pack_start(EQ7k_);
    graphicEQTable_.attach(*manage(new Gtk::Label("7 kHz", 0.0, 0.5)), 2, 3, 10, 11, Gtk::FILL);
    graphicEQTable_.attach(EQBox7k_, 3, 5, 10, 11, Gtk::FILL);

    EQ10k_.set_value_pos(Gtk::POS_RIGHT);
    EQ10k_.set_draw_value();
    EQ10k_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[9] );
    EQ10k_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox10k_.set_spacing(12);
    EQBox10k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox10k_.pack_start(EQ10k_);
    graphicEQTable_.attach(*manage(new Gtk::Label("10 kHz", 0.0, 0.5)), 2, 3, 11, 12, Gtk::FILL);
    graphicEQTable_.attach(EQBox10k_, 3, 5, 11, 12, Gtk::FILL);

    EQ15k_.set_value_pos(Gtk::POS_RIGHT);
    EQ15k_.set_draw_value();
    EQ15k_.set_value( (mainManager_.GetPreference().EfxGraphicEQGain())[10] );
    EQ15k_.set_sensitive(mainManager_.GetPreference().EfxGraphicEQ());
    EQBox15k_.set_spacing(12);
    EQBox15k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox15k_.pack_start(EQ15k_);
    graphicEQTable_.attach(*manage(new Gtk::Label("15 kHz", 0.0, 0.5)), 2, 3, 12, 13, Gtk::FILL);
    graphicEQTable_.attach(EQBox15k_, 3, 5, 12, 13, Gtk::FILL);

    graphicEQTable_.attach(*manage(new Gtk::Label), 0, 5, 13, 19, Gtk::FILL);

    mainNotebook_.append_page(graphicEQTable_, gettext("Equalizer"));

    mainBox_.pack_start(mainNotebook_);

    cancelBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnCancel) );
    mainBtnBox_.pack_start(cancelBtn_);
    okBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWndImpl::OnBtnOk) );
    mainBtnBox_.pack_start(okBtn_);

    mainBox_.pack_start(mainBtnBox_);

    prefWnd_.show_all_children();
}

/// @brief Callback function used when trayicon action option menu is clicked.
void PrefWnd::PrefWndImpl::OnMenuTrayIconAction(const std::string& action)
{
    selectedTrayIconAction_ = action;
}

/// @brief Callback function used when any audio driver check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnAudioDriver(const char* driver)
{
    selectedAudioDriver_ = driver;

    audioAlsa5CardOptionBox_.set_sensitive(false);
    audioAlsa5DevOptionBox_.set_sensitive(false);
    audioAlsa9OptionBox_.set_sensitive(false);
    audioOssOptionBox_.set_sensitive(false);
    audioSunOptionBox_.set_sensitive(false);
    audioAixsOptionBox_.set_sensitive(false);

    if(selectedAudioDriver_ == "alsa") {

	audioAlsa5CardOptionBox_.set_sensitive(true);
	audioAlsa5DevOptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "alsa09") {

	audioAlsa9OptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "oss") {

	audioOssOptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "sun") {

	audioSunOptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "aixs") {

	audioAixsOptionBox_.set_sensitive(true);

    }
}

/// @brief Callback function used when audio driver test button is clicked.
/// @sa MainManager::AudioTestDriver
void PrefWnd::PrefWndImpl::OnBtnAudioDriverTest()
{
    std::map<std::string, std::string> option;

    if(selectedAudioDriver_ == "alsa") {
	option["card"] = audioAlsa5Card_.get_text();
	option["dev"] = audioAlsa5Dev_.get_text();

    } else if(selectedAudioDriver_ == "alsa09") {

	option["dev"] = audioAlsa9Dev_.get_text();	

    } else if(selectedAudioDriver_ == "oss") {

	option["dsp"] = audioOssDsp_.get_text();

    } else if(selectedAudioDriver_ == "sun") {

	option["dev"] = audioSunDev_.get_text();

    } else if(selectedAudioDriver_ == "aixs") {

	option["dev"] = audioAixsDev_.get_text();

    }

    if(mainManager_.AudioTestDriver(selectedAudioDriver_.c_str(), option)) 
	audioDriverTestResultLabel_.set_text(gettext("Success"));
    else
	audioDriverTestResultLabel_.set_text(gettext("Fail"));	

}

/// @brief Callback function used when stereo enhance effector check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnStereoEnhance()
{
    if(!stereoenhanceBtn_.get_active())
	stereoenhanceScale_.set_sensitive(false);
    else
	stereoenhanceScale_.set_sensitive(true);	
}

/// @brief Callback function used when room effector check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnRoom()
{
    if(!roomBtn_.get_active())
	roomScale_.set_sensitive(false);
    else
	roomScale_.set_sensitive(true);
}

/// @brief Callback function used when room2 effector check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnRoom2()
{
    if(!roomBtn2_.get_active())
	room2Scale_.set_sensitive(false);
    else
	room2Scale_.set_sensitive(true);
}

/// @brief Callback function used when harmony effector check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnHarmony()
{
    if(!harmonyBtn_.get_active())
	harmonyScale_.set_sensitive(false);
    else
	harmonyScale_.set_sensitive(true);	
}

/// @brief Callback function used when rhythme effector check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnRhythme()
{
    if(!rhythmeBtn_.get_active())
	rhythmeScale_.set_sensitive(false);
    else
	rhythmeScale_.set_sensitive(true);	
}

/// @brief Callback function used when distortion effector check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnDistortion()
{
    if(!distortionBtn_.get_active())
	distortionScale_.set_sensitive(false);
    else
	distortionScale_.set_sensitive(true);	
}

/// @brief Callback function used when graphic EQ check button is clicked.
void PrefWnd::PrefWndImpl::OnBtnGraphicEQ()
{
    if(!graphicEQBtn_.get_active()) {
	EQ50_.set_sensitive(false);
	EQ100_.set_sensitive(false);
	EQ200_.set_sensitive(false);
	EQ400_.set_sensitive(false);
	EQ800_.set_sensitive(false);
	EQ1500_.set_sensitive(false);
	EQ3k_.set_sensitive(false);
	EQ7k_.set_sensitive(false);
	EQ5k_.set_sensitive(false);
	EQ10k_.set_sensitive(false);
	EQ15k_.set_sensitive(false);
    } else {
	EQ50_.set_sensitive(true);
	EQ100_.set_sensitive(true);
	EQ200_.set_sensitive(true);
	EQ400_.set_sensitive(true);
	EQ800_.set_sensitive(true);
	EQ1500_.set_sensitive(true);
	EQ3k_.set_sensitive(true);
	EQ5k_.set_sensitive(true);
	EQ7k_.set_sensitive(true);
	EQ10k_.set_sensitive(true);
	EQ15k_.set_sensitive(true);
    }
}

/// @brief Callback function used when cancel button is clicked.
void PrefWnd::PrefWndImpl::OnBtnCancel()
{
    prefWnd_.hide();
}

/// @brief Callback function used when ok button is clicked.
///
/// Apply to current Preference and change audio driver if need.
void PrefWnd::PrefWndImpl::OnBtnOk()
{
    Tracer::Trace trace("PrefWnd::PrefWndImpl::OnBtnOk");

    std::string programDir = Glib::get_home_dir();
    programDir += "/.sagua";

    if(!Glib::file_test(programDir, Glib::FILE_TEST_EXISTS)) {	
	// make dir with permission 755
	trace.Debug("programDir does not exist");
	mkdir(programDir.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
	trace.Debug("Create programDir");
    } else if(!Glib::file_test(programDir, Glib::FILE_TEST_IS_DIR)) {
	trace.Debug("File exists already in same name");

	return;
    }

    std::string prefFilename = Glib::get_home_dir();
    prefFilename += "/.sagua/config";

    std::ofstream prefFile(prefFilename.c_str());

    if(prefFile) 
	prefFile << *this; 
    else
	trace.Debug("Can't write prefFile");	

    mainManager_.GetWritablePreference() << prefWnd_;

    if(generalMixerDev_.get_text() != previousMixerDev_)
	mainManager_.MixerRefresh();

    if(mainManager_.GetPreference().Effect())
	mainManager_.GetPreference().EfxChangedSet();

    bool changedAudioDriver = false;

    if(selectedAudioDriver_ != previousAudioDriver_) {

	changedAudioDriver = true;

    } else {

	if(selectedAudioDriver_ == "alsa") {
	    
	    if(audioAlsa5Card_.get_text() != previousAlsa5Card_ ||
		audioAlsa5Dev_.get_text() != previousAlsa5Dev_)
		changedAudioDriver = true;

	} else if(selectedAudioDriver_ == "alsa09") {
	    
	    if(audioAlsa9Dev_.get_text() != previousAlsa9Dev_)
		changedAudioDriver = true;

	} else if(selectedAudioDriver_ == "oss") {

	    if(audioOssDsp_.get_text() != previousOssDsp_)
		changedAudioDriver = true;

	} else if(selectedAudioDriver_ == "sun") {

	    if(audioSunDev_.get_text() != previousSunDev_)
		changedAudioDriver = true;

	} else if(selectedAudioDriver_ == "aixs") {

	    if(audioAixsDev_.get_text() != previousAixsDev_)
		changedAudioDriver = true;

	}
	
    }

    if(changedAudioDriver) {
	trace.Debug("Audio driver changed, i'll try set new audio driver");
	mainManager_.AudioSetChangedDriver();
    }

    if(previousCompactBgPixmap_ != Glib::filename_from_utf8(generalCompactBgPixmap_.GetEntryText()))
	mainManager_.MainWndChangeCompactBgPixmap();

    prefWnd_.hide();
}

/// @brief Apply to Preference.
void operator<<(Preference& preference, const PrefWnd& prefWnd)
{
    preference.generalTrayIconAction_ = prefWnd.impl_->selectedTrayIconAction_;
    preference.generalLeaveOnTrayWhenIconify_ = prefWnd.impl_->generalLeaveOnTrayWhenIconifyBtn_.get_active();
    preference.generalAssumeCharset_ = prefWnd.impl_->generalAssumeCharset_.get_text();
    preference.generalCompactBgPixmap_ = Glib::filename_from_utf8(prefWnd.impl_->generalCompactBgPixmap_.GetEntryText());
    preference.mixerDev_ = prefWnd.impl_->generalMixerDev_.get_text();

    preference.audioDriver_ = prefWnd.impl_->selectedAudioDriver_;
    preference.audioAlsa5Card_ = prefWnd.impl_->audioAlsa5Card_.get_text();
    preference.audioAlsa5Dev_ = prefWnd.impl_->audioAlsa5Dev_.get_text();
    preference.audioAlsa9Dev_ = prefWnd.impl_->audioAlsa9Dev_.get_text();
    preference.audioOssDsp_ = prefWnd.impl_->audioOssDsp_.get_text();
    preference.audioSunDev_ = prefWnd.impl_->audioSunDev_.get_text();
    preference.audioAixsDev_ = prefWnd.impl_->audioAixsDev_.get_text(); 

    preference.efxStereoEnhanceAmount_ = prefWnd.impl_->stereoenhanceScale_.get_value();
    preference.efxRoomAmount_ = prefWnd.impl_->roomScale_.get_value();
    preference.efxRoom2Amount_ = prefWnd.impl_->room2Scale_.get_value();
    preference.efxHarmonyAmount_ = prefWnd.impl_->harmonyScale_.get_value();
    preference.efxRhythmeAmount_ = prefWnd.impl_->rhythmeScale_.get_value();
    preference.efxDistortionAmount_ = prefWnd.impl_->distortionScale_.get_value();
    preference.efxBassBoost_ = prefWnd.impl_->bassboostBtn_.get_active();
    preference.efxBassBoost2_ = prefWnd.impl_->bassboost2Btn_.get_active();
    preference.efxStereoEnhance_ = prefWnd.impl_->stereoenhanceBtn_.get_active();
    preference.efxTrebleBoost_ = prefWnd.impl_->trebleboostBtn_.get_active();
    preference.efxRoom_ = prefWnd.impl_->roomBtn_.get_active();
    preference.efxRoom2_ = prefWnd.impl_->roomBtn2_.get_active();
    preference.efxHarmony_ = prefWnd.impl_->harmonyBtn_.get_active();
    preference.efxRhythme_ = prefWnd.impl_->rhythmeBtn_.get_active();
    preference.efxDistortion_ = prefWnd.impl_->distortionBtn_.get_active();
    preference.efxGraphicEQ_ = prefWnd.impl_->graphicEQBtn_.get_active();

    preference.effect_ = preference.efxBassBoost_ || preference.efxBassBoost2_ || preference.efxStereoEnhance_ \
			 || preference.efxTrebleBoost_ || preference.efxRoom_ || preference.efxRoom2_\
			 || preference.efxHarmony_ || preference.efxRhythme_ \
			 || preference.efxDistortion_ || preference.efxGraphicEQ_;

    preference.efxGraphicEQGain_[0] = prefWnd.impl_->EQ50_.get_value();		    
    preference.efxGraphicEQGain_[1] = prefWnd.impl_->EQ100_.get_value();		    
    preference.efxGraphicEQGain_[2] = prefWnd.impl_->EQ200_.get_value();		    
    preference.efxGraphicEQGain_[3] = prefWnd.impl_->EQ400_.get_value();		    
    preference.efxGraphicEQGain_[4] = prefWnd.impl_->EQ800_.get_value();		    
    preference.efxGraphicEQGain_[5] = prefWnd.impl_->EQ1500_.get_value();		    
    preference.efxGraphicEQGain_[6] = prefWnd.impl_->EQ3k_.get_value();		    
    preference.efxGraphicEQGain_[7] = prefWnd.impl_->EQ5k_.get_value();		    
    preference.efxGraphicEQGain_[8] = prefWnd.impl_->EQ7k_.get_value();		    
    preference.efxGraphicEQGain_[9] = prefWnd.impl_->EQ10k_.get_value();		    
    preference.efxGraphicEQGain_[10] = prefWnd.impl_->EQ15k_.get_value();		    
}

/// @brief Output to ostream, generally file.
void operator<<(std::ostream& os, const PrefWnd::PrefWndImpl& prefWndImpl)
{
    bool effect =  prefWndImpl.bassboostBtn_.get_active() || prefWndImpl.bassboost2Btn_.get_active() || prefWndImpl.trebleboostBtn_.get_active() \
		   || prefWndImpl.stereoenhanceBtn_.get_active() || prefWndImpl.roomBtn_.get_active() || prefWndImpl.roomBtn2_.get_active() \
		   || prefWndImpl.harmonyBtn_.get_active() || prefWndImpl.rhythmeBtn_.get_active() \
		   || prefWndImpl.distortionBtn_.get_active() || prefWndImpl.graphicEQBtn_.get_active();

    os << "GeneralTrayIconAction = " << prefWndImpl.selectedTrayIconAction_ << std::endl;
    os << "GeneralLeaveOnTrayWhenIconify = " << prefWndImpl.generalLeaveOnTrayWhenIconifyBtn_.get_active() << std::endl;
    os << "GeneralAssumeCharset = " << prefWndImpl.generalAssumeCharset_.get_text() << std::endl;
    os << "GeneralCompactBgPixmap = " << Glib::filename_from_utf8(prefWndImpl.generalCompactBgPixmap_.GetEntryText()) << std::endl;
    os << "AudioDriver = " << prefWndImpl.selectedAudioDriver_ << std::endl;
    os << "AudioAlsa5Card = " << prefWndImpl.audioAlsa5Card_.get_text() << std::endl;
    os << "AudioAlsa5Dev = " << prefWndImpl.audioAlsa5Dev_.get_text() << std::endl;
    os << "AudioAlsa9Dev = " << prefWndImpl.audioAlsa9Dev_.get_text() << std::endl;
    os << "AudioOssDsp = " << prefWndImpl.audioOssDsp_.get_text() << std::endl;
    os << "AudioSunDev = " << prefWndImpl.audioSunDev_.get_text() << std::endl;
    os << "AudioAixsDev = " << prefWndImpl.audioAixsDev_.get_text() << std::endl;
    os << "MixerDev = " << prefWndImpl.generalMixerDev_.get_text() << std::endl;
    os << "Effect = " << effect << std::endl;			
    os << "EfxBassBoost = " << prefWndImpl.bassboostBtn_.get_active() << std::endl;
    os << "EfxBassBoost2 = " << prefWndImpl.bassboost2Btn_.get_active() << std::endl;
    os << "EfxTrebleBoost = " << prefWndImpl.trebleboostBtn_.get_active() << std::endl;
    os << "EfxStereoEnhance = " << prefWndImpl.stereoenhanceBtn_.get_active() << std::endl;
    os << "EfxStereoEnhanceAmount = " << prefWndImpl.stereoenhanceScale_.get_value() << std::endl;
    os << "EfxRoom = " << prefWndImpl.roomBtn_.get_active() << std::endl;
    os << "EfxRoom2 = " << prefWndImpl.roomBtn2_.get_active() << std::endl;
    os << "EfxRoomAmount = " << prefWndImpl.roomScale_.get_value() << std::endl;
    os << "EfxRoom2Amount = " << prefWndImpl.room2Scale_.get_value() << std::endl;
    os << "EfxHarmony = " << prefWndImpl.harmonyBtn_.get_active() << std::endl;
    os << "EfxHarmonyAmount = " << prefWndImpl.harmonyScale_.get_value() << std::endl;
    os << "EfxRhythme = " << prefWndImpl.rhythmeBtn_.get_active() << std::endl;
    os << "EfxRhythmeAmount = " << prefWndImpl.rhythmeScale_.get_value() << std::endl;
    os << "EfxDistortion = " << prefWndImpl.distortionBtn_.get_active() << std::endl;
    os << "EfxDistortionAmount = " << prefWndImpl.distortionScale_.get_value() << std::endl;
    os << "EfxGraphicEQ = " << prefWndImpl.graphicEQBtn_.get_active() << std::endl;
    os << "EfxGraphicEQGain50 = " << prefWndImpl.EQ50_.get_value() << std::endl;
    os << "EfxGraphicEQGain100 = " << prefWndImpl.EQ100_.get_value() << std::endl;
    os << "EfxGraphicEQGain200 = " << prefWndImpl.EQ200_.get_value() << std::endl;
    os << "EfxGraphicEQGain400 = " << prefWndImpl.EQ400_.get_value() << std::endl;
    os << "EfxGraphicEQGain800 = " << prefWndImpl.EQ800_.get_value() << std::endl;
    os << "EfxGraphicEQGain1.5k = " << prefWndImpl.EQ1500_.get_value() << std::endl;
    os << "EfxGraphicEQGain3k = " << prefWndImpl.EQ3k_.get_value() << std::endl;
    os << "EfxGraphicEQGain5k = " << prefWndImpl.EQ5k_.get_value() << std::endl;
    os << "EfxGraphicEQGain7k = " << prefWndImpl.EQ7k_.get_value() << std::endl;
    os << "EfxGraphicEQGain10k = " << prefWndImpl.EQ10k_.get_value() << std::endl;
    os << "EfxGraphicEQGain15k = " << prefWndImpl.EQ15k_.get_value() << std::endl;
}

